#include <windows.h>
#include <httpfilt.h>

/*
  Scrive il contatore nella stringa formattandolo in
  modo acconcio
*/
void FormattaContatore( DWORD numero,
                        char  *stringa,
                        DWORD caratteri)
{
  char cNull = '0';
  DWORD n, i;
  
  n = numero;
  i = caratteri;

  /* scrivo i singoli caratteri */
  while (i)
  {
    *(stringa+(--i)) = (LOBYTE (LOWORD (n % 10))) + cNull;
    if( !(n /= 10) )
    {
      cNull = ' ';
    }
  }
}

/*
  Tiene il conto degli eventi che si verificano nel nostro sito Web.
  Il conteggio viene scritto in un file .INI denominato "CONTATORE.INI"
  che si trova nella stessa directory della DLL. Gli eventi sono inseriti
  nella sezione del file denominata [EVENTI].
*/
DWORD ContaEventi(char *evento )
{
  UINT conteggio;
  char stringa[11];
  char *nomeIni = "CONTATORE.INI";
  char *sezioneEventi = "EVENTI";

  /* leggo il conteggio attuale */
  conteggio = GetPrivateProfileInt( sezioneEventi,
                                    evento,
                                    0,
                                    nomeIni );

  /* incremento il conteggio e mi preparo a riscriverlo */
  wsprintf( stringa, "%u", ++conteggio );
  
  /* scrivo il conteggio nel file */
  WritePrivateProfileString( sezioneEventi, evento,
                             stringa, nomeIni );

  /* ritorno il conteggio attuale */
  return conteggio;

}

/*
  Inserisce il contatore nella pagina HTML.
  La funzione cerca la sequenza &lt!--!!!--&gt e la sostituisce con il valore
  indicato come conteggio.
  numDati indica il numero di caratteri che compongono la pagina.
*/
void ScriviConteggio( DWORD contatore, void *dati, DWORD numDati )
{
  char  *pagina;
  DWORD i, j;
  char *indicatore = "&lt!--!!!--&gt";

  /* trasformo i dati in caratteri */  
  pagina = dati;
  
  /* ciclo e mi leggo tutti i dati */
  i = 0;
  while (i != numDati)
  {
    /* verifica se quello che sto' leggendo e' la sequenza "indicatore" */
    j = 0;
    while( ( (i+j) != numDati) &&
             indicatore[j] && (*(pagina+i+j) == indicatore[j]))
    {
      j++;
    }

    /* ho trovato l'indicatore ? */
    if( indicatore[j] )
    {
      /* no */
      i++;
    }
    else
    {
      /* SI, ficca dentro il contatore */
      FormattaContatore( contatore, pagina+i, j);
      i += j;
    }
  }
}

/*
* Inizializzazione del filtro.
* la struttura HTTP_FILTER_VERSION viene passata da IIS e deve essere
* riempita prima di essere ritornata
*/
BOOL WINAPI GetFilterVersion (HTTP_FILTER_VERSION *fv)
{
  /* HTTP_FILTER_REVISION e' una costante definita in httpfilt.h */
  fv->dwFilterVersion = HTTP_FILTER_REVISION;

  /* inserisco una descrizione per il contatore */  
  lstrcpyn( fv->lpszFilterDesc, "Contatore Scemo Numero 1",
            SF_MAX_FILTER_DESC_LEN);
  
  /* imposto gli eventi di cui voglio sapere */
  fv->dwFlags = (SF_NOTIFY_URL_MAP            |
                 SF_NOTIFY_SEND_RAW_DATA      |
                 SF_NOTIFY_END_OF_NET_SESSION |
                 SF_NOTIFY_ORDER_LOW          |
                 SF_NOTIFY_NONSECURE_PORT);

  return TRUE;
}


/*
  Processa le operazioni del filtro
  Questa funzione viene chiamata da IIS ogni volta che si verifica un
  evento di quelli indicati dalla GetFilterVersion()
  
  Gli eventi gestiti in questa procedura sono: la richiesta di un documento,
  l'abbandono del sito da parte dell'utente e l'invio di un documento al
  browser dell'utente.
  
*/
DWORD WINAPI HttpFilterProc (HTTP_FILTER_CONTEXT *fc,
                             DWORD notificationType,
                             VOID  *notification)
{
  
  HTTP_FILTER_URL_MAP  *urlMap;     /* il documento richiesto */
  HTTP_FILTER_RAW_DATA *rawData;    /* la pagina pronta da spedire */
  
  switch (notificationType)
  {
    case SF_NOTIFY_URL_MAP:
      /*
        l'utente ha richiesto un documento, incremento il contatore di
        richieste per quel documento
      */
      urlMap = notification;
      fc->pFilterContext = (PVOID) ContaEventi( urlMap->pszPhysicalPath );
      break;

    case SF_NOTIFY_SEND_RAW_DATA:
      /*
        Stiamo per inviare una pagina, devo aggiornare il contatore sulla
        pagina prima che questa sia spedita al client
      */
      rawData = notification;
      ScriviConteggio( (DWORD) fc->pFilterContext, rawData->pvInData,
        rawData->cbInData);
      break;

    case SF_NOTIFY_END_OF_NET_SESSION:
      /*
        L'utente se ne' andato, aumento il contatore di utenti che
        hanno visitato il sito
      */
      ContaEventi( "FINE_SESSIONI" );
      break;

    default:
      /* non gestisco altri eventi */
      break;
  }

  /* voglio ricevere altri eventi ? SIIIII!!! */
  return SF_STATUS_REQ_NEXT_NOTIFICATION;

}

